//STM32 Driver for LCD NOKIA5110 PCD8544 monochrome 84x48 pixels
//Made by Nanoslavic https://www.youtube.com/channel/UCunTBIPyQJXAYcR-3R2wyFw/
//Драйвер написал Нанославик https://www.youtube.com/channel/UCV-6hEx_fxkrsyH0Jvte49w/

//===========================================================================
//Main LCD functions (Основные функции управления дисплеем)

//Pixel buffer for drawing (Буфер для рисования)
uint8_t GraphicBuffer[84*6];

//Init pins and LCD (инициализация пинов и дисплея)
extern void lcd_Init(void);

//Write buffer to LCD (Выводит буфер в дисплей)
extern void lcd_Update();

//Set pixel in the buffer (Включает пиксель в буфере по координатам)
extern void lcd_PixelSet(int32_t x, int32_t y);

//Clear pixel in the buffer (Отключает пиксель в буфере по координатам)
extern void lcd_PixelClear(int32_t x, int32_t y);

//Clear graphic buffer (очищает графический буфер)
extern void lcd_Clear(void);

//Fill buffer by data byte (заполняет буфер байтом данных)
extern void lcd_Fill(uint8_t data);

//===========================================================================
//Pin configuration (Назначение выводов для управления дисплеем)
/*
Backlight control (управление подсветкой)
For the red module, the backlight is turned on by shorting the pin to ground. For the blue module, the backlight is turned on by shorting the pin to +3.3V.
(У красного модуля подсветка включается замыканием пина на землю. У синего модуля подсветка включается замыканием пина на плюс питания)
*/

//LCD Pins
//LCD Chip enable pin
#define N5110d_CE_GPIO		GPIOA
#define N5110d_CE_BIT		GPIO_Pin_3
#define N5110d_CE_RCC		RCC_AHB1Periph_GPIOA

//LCD Reset pin
#define N5110d_RST_GPIO		GPIOA
#define N5110d_RST_BIT		GPIO_Pin_4
#define N5110d_RST_RCC		RCC_AHB1Periph_GPIOA

//LCD Command-data pin
#define N5110d_DC_GPIO		GPIOA
#define N5110d_DC_BIT		GPIO_Pin_2
#define N5110d_DC_RCC		RCC_AHB1Periph_GPIOA

//LCD Clock in pin
#define N5110d_CLK_GPIO		GPIOA
#define N5110d_CLK_BIT		GPIO_Pin_0
#define N5110d_CLK_RCC		RCC_AHB1Periph_GPIOA

//LCD Data in pin
#define N5110d_DIN_GPIO		GPIOA
#define N5110d_DIN_BIT		GPIO_Pin_1
#define N5110d_DIN_RCC		RCC_AHB1Periph_GPIOA

//===========================================================================
//Next low level functions (дальше реализация низкоуровневых функций)

//Activate LCD interface
void N5110_CE_ON(void){
	GPIO_ClearBit(N5110d_CE_GPIO,N5110d_CE_BIT);
}

//Disable LCD interface
void N5110_CE_OFF(void){
	GPIO_SetBit(N5110d_CE_GPIO,N5110d_CE_BIT);
}

//Clock bit
static void N5110_Clock(void){
	GPIO_ClearBit(N5110d_CLK_GPIO,N5110d_CLK_BIT);
	GPIO_SetBit(N5110d_CLK_GPIO,N5110d_CLK_BIT);
}

//Write 1 byte to LCD
void N5110_OutByte(uint8_t d){
if ((d&(1<<7))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
if ((d&(1<<6))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
if ((d&(1<<5))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
if ((d&(1<<4))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
if ((d&(1<<3))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
if ((d&(1<<2))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
if ((d&(1<<1))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
if ((d&(1<<0))>0) GPIO_SetBit(N5110d_DIN_GPIO,N5110d_DIN_BIT); else GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
N5110_Clock();
}

//Send command byte to LCD
void N5110_SendCom(uint8_t com){
	N5110_CE_ON();
	GPIO_ClearBit(N5110d_DC_GPIO,N5110d_DC_BIT);
	N5110_OutByte(com);
	N5110_CE_OFF();
}

//Send data byte to LCD
void N5110_SendData(uint8_t com){
	N5110_CE_ON();
	GPIO_SetBit(N5110d_DC_GPIO,N5110d_DC_BIT);
	N5110_OutByte(com);
	N5110_CE_OFF();
}

//Reset LCD
void N5110_Reset(void){
	GPIO_ClearBit(N5110d_RST_GPIO,N5110d_RST_BIT);
	delay_ms(30);
	GPIO_SetBit(N5110d_RST_GPIO,N5110d_RST_BIT);
	delay_ms(20);
}

//Init pins and LCD (инициализация пинов и дисплея)
void lcd_Init(void){
	AHB1_ClockEnable(N5110d_RST_RCC|N5110d_CE_RCC|N5110d_DC_RCC|N5110d_CLK_RCC|N5110d_DIN_RCC);
	init_pin_opp(N5110d_RST_GPIO,N5110d_RST_BIT);
	init_pin_opp(N5110d_CE_GPIO,N5110d_CE_BIT);
	init_pin_opp(N5110d_DC_GPIO,N5110d_DC_BIT);
	init_pin_opp(N5110d_CLK_GPIO,N5110d_CLK_BIT);
	init_pin_opp(N5110d_DIN_GPIO,N5110d_DIN_BIT);

	GPIO_SetBit(N5110d_RST_GPIO,N5110d_RST_BIT);
	GPIO_SetBit(N5110d_CE_GPIO,N5110d_CE_BIT);
	GPIO_ClearBit(N5110d_DC_GPIO,N5110d_DC_BIT);
	GPIO_ClearBit(N5110d_DIN_GPIO,N5110d_DIN_BIT);
	GPIO_SetBit(N5110d_CLK_GPIO,N5110d_CLK_BIT);

	//Reset LCD
	N5110_Reset();

	//Init LCD
	N5110_SendCom(0x21);// Function set. Extended instruction set control ON
	N5110_SendCom(0xB9);// Set LCD Contrast 0xB1 or 0xBF (настройка контраста от 0xB1 до 0xBF)
	N5110_SendCom(0x04);// Set Temp coefficient
	N5110_SendCom(0x14);// LCD bias mode 1:48 (0x12, 0x13, 0x14)
	N5110_SendCom(0x20);// Function set. Extended instruction set control OFF
	N5110_SendCom(0x0C);// Set normal mode (нормальный режим отображения)
	//N5110_SendCom(0x0D);// Set inverse mode (инверсный режим отображения)
}

//===========================================================================
//Drawing
//===========================================================================

/*
1 byte for 8 vertical pixels
Up to down, left to right

Расположение пикселей на дисплее:
Pixel arrangement on the display:

BYTE0_BIT0(x0,y0)	BYTE1_BIT0(x1,y0)	BYTE2_BIT0(x2,y0)	BYTE3_BIT0(x3,y0)	...
BYTE0_BIT1(x0,y1)	BYTE1_BIT1(x1,y1)	BYTE2_BIT1(x2,y1)	BYTE3_BIT1(x3,y1)	...
BYTE0_BIT2(x0,y2)	BYTE1_BIT2(x1,y1)	BYTE2_BIT2(x2,y2)	BYTE3_BIT2(x3,y1)	...
BYTE0_BIT3(x0,y3)	BYTE1_BIT3(x1,y1)	BYTE2_BIT3(x2,y3)	BYTE3_BIT3(x3,y1)	...
....				...					...					...
*/

//Write buffer to LCD (Выводит буфер в дисплей)
void lcd_Update(){
	uint16_t i;

	N5110_SendCom(0x80);// Column 0 to 83 (0x80 to 0xD3)
	N5110_SendCom(0x40);// Row 0 to 5 (0x40...0x45)

	for (i=0; i < (84*6); i++){
		N5110_SendData(GraphicBuffer[i]);
	};
}

//Fill buffer by data byte (заполняет буфер байтом данных)
void lcd_Fill(uint8_t data){
	uint16_t i;

	for (i = 0; i < (84*6); i++){
		GraphicBuffer[i] = data;
	};
}

//Clear graphic buffer (очищает графический буфер)
void lcd_Clear(void){
	uint16_t i;

	for (i = 0; i < (84*6); i++){
		GraphicBuffer[i] = 0;
	};
}


//Set pixel in the buffer (Включает пиксель в буфере по координатам)
void lcd_PixelSet(int32_t x, int32_t y){
	if (x<0) return;
	if (y<0) return;
	if (x>83) return;
	if (y>47) return;
	uint32_t tmp=y/8*84 + x;
	GraphicBuffer[tmp]=GraphicBuffer[tmp]|(1<<(y%8));
}

//Clear pixel in the buffer (Отключает пиксель в буфере по координатам)
void lcd_PixelClear(int32_t x, int32_t y){
	if (x<0) return;
	if (y<0) return;
	if (x>83) return;
	if (y>47) return;
	uint32_t tmp=y/8*84 + x;
	GraphicBuffer[tmp]=GraphicBuffer[tmp]&(~(1<<(y%8)));
}

